#include <cstdlib>
#include <fstream>
#include <iostream>

#include <gsl/gsl_math.h>
#include "IOUtilities.h"
using namespace std;

bool SaveData(const DBL_VCTR_2D& Xtild, const INT_VCTR_3D& X, const string& File)
{
	if (Xtild.size() == 0)
	{
		cerr << "Uninitialized Xtild - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	if (Xtild.size() < X.size())
	{
		cerr << "Mismatched Xtild and X - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	int i, a,l,b;

	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cerr << "Cannot open file " << File << " - SaveData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(Xtild.size());
	int B = static_cast<int>(Xtild[0].size());
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());

	for (i = 0; i < M; i++)
	{
		for (a = 0; a < A; a++)
		{
			for (b=1;b < B;b++)
			{
				ofs << Xtild[i][b] << "\t";
			}
			if (i < N)
			{
				for (l = 0; l < L; l++)
				{
					ofs << X[i][a][l] << "\t";
				}
			}
			else
			{
				for (l = 0; l < L; l++)
				{
					ofs << 0 << "\t";
				}
			}
			ofs << endl;
		}
	}

	ofs.close();

	return true;
}


void SaveBetaToMyFile(const DBL_VCTR_3D& Beta, const string& File)
{
	int nb_cycles=(int)Beta.size();
	int K=(int)Beta[0].size();
	int P=(int)Beta[0][0].size();
	int cycle,k,p;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cout << "Cannot open file " << File << " - SaveBetaToMyFile" << endl;
		exit(1);
	}
	ofs << nb_cycles << "\t" << K << "\t" << P << "\t" << endl;
	for (cycle=0; cycle < nb_cycles; cycle++)
	{
		for (k=0; k < K; k++)
		{
			for (p=0; p < P; p++)
			{
   
				ofs << Beta[cycle][k][p] << "\t";
			}
		}
		ofs << endl;
	}
	ofs.close();
// 	cout<<"Beta saved in "<<File<<endl;
} 

bool LoadData(DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, INT_VCTR_3D& X, int ER, int EC, const string& File, const bool& specialFormat, const int& nb_var_regr)
{
	if (Xtild.size() == 0)
	{
		cerr << "Uninitialized Xtild - LoadData" << endl;
		exit(EXIT_FAILURE);
	}
	
	if (C.size() == 0)
	{
		cerr << "Uninitialized C - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (X.size() == 0)
	{
		cerr << "Uninitialized X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	if (Xtild.size() < X.size())
	{
		cerr << "Mismatched Xtild and X - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int i, a, l, b;
	
	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadData" << endl;
		exit(EXIT_FAILURE);
	}

	int M = static_cast<int>(Xtild.size());
	int B = static_cast<int>(Xtild[0].size());
	int N = static_cast<int>(X.size());
	int A = static_cast<int>(X[0].size());
	int L = static_cast<int>(X[0][0].size());
	double x,y;
	int nbDigitMax = 0;

	// Load and discard the extra rows.
	char extraRow[10240];
	for (int er = 0; er < ER; er++)
	{
		ifs.getline(extraRow, 10240);
	}
	
	if (!specialFormat)
	{
		for (i = 0; i < M; i++)
		{
			for (a = 0; a < A; a++)
			{
				// Load and discard the extra columns.
				string discarded;
				for (int ec = 0; ec < EC; ec++)
				{
					ifs >> discarded;
				}
			
				ifs >> C[i][0];
				ifs >> C[i][1];
			
				x = C[i][0];
				y = C[i][1];
						
				if (nb_var_regr == 1)
				{
					// We keep only y (ignoring leftmost column)
					for (b = 1; b < B; b ++)
					{
						Xtild[i][b] = gsl_pow_int(y,b);
					}
				}	
				
				if (nb_var_regr == 2)
				{
					//In any case, B is one of 1,3,6,10
					if (B > 1) 
					{
						Xtild[i][1] = x;
						Xtild[i][2] = y;
						if (B > 3)
						{
							Xtild[i][3] = x*x;
							Xtild[i][4] = y*y;
							Xtild[i][5] = x*y;
						
							if (B > 6)
							{
								Xtild[i][6] = x*x*x;
								Xtild[i][7] = y*y*y;
								Xtild[i][8] = x*x*y;
								Xtild[i][9] = x*y*y;
							}
						}
					}
				}
				if (i < N)
				{
					for (l = 0; l < L; l++)
					{
						ifs >> X[i][a][l];
					}
				}
				else
				{
					for (l = 0; l < L; l++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}
	if (specialFormat)
	{
		for (i = 0; i < M; i ++)
		{
			// Load and discard the extra columns.
			string discarded;
			for (int ec = 0; ec < EC; ec++)
			{
				ifs >> discarded;
			}
                        		
			ifs >> C[i][0] >> C[i][1];
			
			if (i < N)
			{
				
				x = C[i][0];
				y = C[i][1];
						
				if (nb_var_regr == 1)
				{
					// We keep only y (ignoring leftmost column)
					for (b = 1; b < B; b ++)
					{
						Xtild[i][b] = gsl_pow_int(b,y);
					}
				}	
				
				if (nb_var_regr == 2)
				{
					//In any case, B is one of 1,3,6,10
					if (B > 1) 
					{
						Xtild[i][1] = x;
						Xtild[i][2] = y;
						if (B > 3)
						{
							Xtild[i][3] = x*x;
							Xtild[i][4] = y*y;
							Xtild[i][5] = x*y;
						
							if (B > 6)
							{
								Xtild[i][6] = x*x*x;
								Xtild[i][7] = y*y*y;
								Xtild[i][8] = x*x*y;
								Xtild[i][9] = x*y*y;
							}
						}
					}
				}
				
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> X[i][a][l];
					}
				}
			} else
			{
				for (l = 0; l < L; l++)
				{
					for (a = 0; a < A; a ++)
					{
						ifs >> discarded;
					}
				}
			}
		}
	}

	ifs.close();
	
		

	return true;
}


bool SaveCF(const DBL_VCTR_1D& sigma, const DBL_VCTR_2D& beta, const DBL_VCTR_2D& alpha, const DBL_VCTR_2D& q, const INT_VCTR_3D& zadmix, const DBL_VCTR_3D& p, const string& File)
{
	
	if (beta.size() == 0)
	{
		cerr << "Uninitialized beta - SaveCF" << endl;
		return false;	
	}
	
	if (q.size() == 0)
	{
		cerr << "Uninitialized Q - SaveCF" << endl;
		return false;	
	}
	
	if (zadmix.size() == 0)
	{
		cerr << "Uninitialized zadmix - SaveCF" << endl;
		return false;
	}

	if (p.size() == 0)
	{
		cerr << "Uninitialized p - SaveCF" << endl;
		return false;
	}

	int i, k, l, j, b,a;

	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cerr << "Cannot open file " << File << " - SaveCF" << endl;
		return false;
	}

	// K, L, J
	int K = static_cast<int>(p.size());
	int L = static_cast<int>(p[0].size());
	INT_VCTR_1D J(L, 0);
	for (l = 0; l < L; l++)
	{
		J[l] = static_cast<int>(p[0][l].size());
	}
	
	ofs << K << ' ';
	ofs << L << ' ';
	for (l = 0; l < L; l++)
	{
		ofs << J[l] << ' ';
	}

	// N,A
	int N = static_cast<int>(zadmix.size());
	int A = static_cast<int>(zadmix[0].size());
	ofs << N << ' ';

	// beta_length
	int B = static_cast<int>(beta[0].size());
	
	// p
	for (k = 0; k < K; k++)
	{
		for (l = 0; l < L; l++)
		{
			for (j = 0; j < J[l]; j++)
			{
				ofs << p[k][l][j] << ' ';
			}
		}
	}
	
	// zadmix
	for (i = 0; i < N; i++)
	{
		for (a=0;a<A;a++)
		{
			for (l=0;l<L;l++)
			{
				ofs << zadmix[i][a][l] << ' ';
			}
		}
	}

        //q
	for (i=0;i<N;i++)
	{
		for (k=0;k<K;k++)
		{
			ofs << q[i][k] << ' ';
		}
	}
	
	//alpha
	for (i=0;i<N;i++)
	{
		for (k=0;k<K;k++)
		{
			ofs << alpha[i][k] << ' ';
		}
	}
	
	//beta
	for (k = 0; k < K; k ++)
	{
		for (b = 0; b < B; b ++)
		{
			ofs << beta[k][b] << ' ';
		}
	}
	
	//sigma
	for (k = 0; k < K; k ++)
	{
		ofs << sigma[k] << ' ';
	}

	ofs.close();

	return true;
}

bool LoadCF(DBL_VCTR_1D& sigma, DBL_VCTR_2D& beta, DBL_VCTR_2D& alpha, DBL_VCTR_2D& q, INT_VCTR_3D& zadmix, DBL_VCTR_3D& p, const string& File)
{
	
	if (beta.size() == 0)
	{
		cerr << "Uninitialized beta - LoadCF" << endl;
		return false;	
	}
	
	if (q.size() == 0)
	{
		cerr << "Uninitialized Q - LoadCF" << endl;
		return false;	
	}
	
	if (zadmix.size() == 0)
	{
		cerr << "Uninitialized zadmix - LoadCF" << endl;
		return false;
	}

	if (p.size() == 0)
	{
		cerr << "Uninitialized p - LoadCF" << endl;
		return false;
	}

	int i, k, l, j,a,b;

	// K, L, J
	int K = static_cast<int>(p.size());
	int L = static_cast<int>(p[0].size());
	INT_VCTR_1D J(L, 0);
	for (l = 0; l < L; l++)
	{
		J[l] = static_cast<int>(p[0][l].size());
	}

	// N,A
	int N = static_cast<int>(zadmix.size());
	int A = static_cast<int>(zadmix[0].size());

	//B
	int B = static_cast<int>(beta[0].size());
	
	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadCF" << endl;
		return false;
	}

	// KF, LF, JF
	int KF;
	int LF;
	INT_VCTR_1D JF;
	ifs >> KF;
	ifs >> LF;
	for (l = 0; l < LF; l++)
	{
		JF.push_back(0);
		ifs >> JF[l];
	}

	// NF
	int NF;
	ifs >> NF;
	if (KF != K || LF != L || JF != J || NF != N)
	{
		ifs.close();

		cerr << "CF file conflicts with command-line options" << " - LoadCF" << endl;
		return false;
	}

	for (k = 0; k < KF; k++)
	{
		for (l = 0; l < LF; l++)
		{
			for (j = 0; j < JF[l]; j++)
			{
				ifs >> p[k][l][j];
			}
		}
	}

	// zadmix
	for (i = 0; i < N; i++)
	{
		for (a=0;a<A;a++)
		{
			for (l=0;l<L;l++)
			{
				ifs >> zadmix[i][a][l];
			}
		}
	}

        //q
	for (i=0;i<N;i++)
	{
		for (k=0;k<K;k++)
		{
			ifs >> q[i][k];
		}
	}
	
	//alpha
	for (i=0;i<N;i++)
	{
		for (k=0;k<K;k++)
		{
			ifs >> alpha[i][k];
		}
	}
	
	//beta
	for (k = 0; k < K; k ++)
	{
		for (b = 0; b < B; b ++)
		{
			ifs >> beta[k][b];
		}
	}
	
	//sigma
	for (k = 0; k < K; k ++)
	{
		ifs >> sigma[k];
	}
	
	ifs.close();


	return true;
}

bool SaveML(const int& trendDeg, const double& minDIC,const string& File)
{
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cerr << "Cannot open file " << File << " - SaveML" << endl;
		return false;
	}
	ofs << minDIC << ' ';
	ofs << trendDeg << ' ';
	ofs << "1" << ' '; 	//admixture = true
	ofs.close();
	return true;
}

bool LoadML(int& trendDeg, double& minDIC ,const string& File,bool quiet)
{
	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		if (!quiet)
		{
			cerr << "Cannot open file " << File << " - LoadML" << endl;
		}
		return false;
	}
	
	ifs >> minDIC ;
	ifs >> trendDeg;
	ifs.close();
	return true;
}
